# Copyright (c) Stephan Martin <sm@sm-zone.net>
#
# $Id: GUI.pm,v 1.12 2003/06/23 21:31:17 sm Exp $
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.

use strict;
package GUI;
require Tk::DialogBox;
require Tk::NoteBook;
require Tk::HList;
require Tk::ItemStyle;

### create the main object
sub new {
   my $that = shift;
   my $class = ref($that) || $that;

   my $self = {};
   $self->{'init'} = shift;

   bless($self, $class);

   $self->{'version'} = '0.4.6 (beta)';

   # initialize CA object
   $self->{'CA'} = CA->new($self->{'init'});

   # initialize OpenSSL object
   $self->{'OpenSSL'} = OpenSSL->new($self);

   # initialize CERT object
   $self->{'CERT'} = CERT->new();

   # initialize KEY object
   $self->{'KEY'} = KEY->new();

   # initialize REQ object
   $self->{'REQ'} = REQ->new();

   # initialize CONFIG object
   $self->{'CONFIG'} = TCONFIG->new();

   # initialize main window
   $self->{'mw'} = MainWindow->new();
   $self->{'mw'}->title("Tiny CA Management $self->{'version'}");
   $self->{'mw'}->minsize(qw(640 480));
   $self->{'mw'}->geometry('640x480');

   $self->create_menu();
   $self->act_menu();

   $self->{'mw'}->fontCreate(
         'small',
         -family => 'helvetica',
         -size   => 11
         );

   $self->{'mw'}->fontCreate(
         'medium',
         -family => 'helvetica',
         -size   => 12,
         -weight => 'bold'
         );

   $self->{'mw'}->fontCreate(
         'big',
         -family => 'helvetica',
         -size   => 14,
         -weight => 'bold'
         );

   if(@{$self->{'CA'}->{'calist'}}) {
      $self->{'CA'}->open_ca($self);
   }
   else {
      $self->{'CA'}->create_ca($self);
   }

   return;
}

### create/update the main frame with the notebook tab
sub create_mframe {
   my $self = shift;

   my($parsed, $ca_left, $ca_right, $cert_list_frame, $req_list_frame,
      $key_list_frame, $line, $state, $button);

   return if not defined($self->{'CA'}->{'actca'});
   return if $self->{'CA'}->{'actca'} eq "";
   my $ca = $self->{'CA'}->{'actca'};

   $parsed = $self->{'CERT'}->parse_cert(
         $self,
         'CA'
         );

   defined($parsed) || $self->print_error("can't read CA Certificate");

   ### frame for whole notebook (all of the screen)
   if(not defined $self->{'mframe'}) {
      $self->{'mframe'} = $self->{'mw'}->Frame(
         -relief      => 'groove',
         -borderwidth => 2,
         )->pack(
            -side   => 'top',
            -expand => 1,
            -fill   => 'both'
            );
   }

   if(not defined $self->{'nb'}) {
      $self->{'nb'} = $self->{'mframe'}->NoteBook( 
         -ipadx => 6,
         -ipady => 6
         );
   }

   ### notebooktab for ca information (split info and buttons)
   if(not defined($self->{'catab'})) {
      $self->{'catab'} = $self->{'nb'}->add(
         'CA',
         -label     => 'CA',
         -underline => 1
         );
   }
   
   ## frame for CA information
   if(not defined $self->{'catab'}->{'top'}) {
      $self->{'catab'}->{'top'} = $self->{'catab'}->Frame(
         -relief      => 'groove',
         -borderwidth => 2,
         -background  => 'white'
         )->pack(
            -side   => 'top',
            -expand => 1,
            -fill   => 'both'
            );
   } else {
      foreach($self->{'catab'}->{'top'}->packSlaves()) {
         $_->destroy();
      }
   }

   $self->{'catab'}->{'top'}->Label(
         -text => "Actual CA: $ca",
         -font => 'big',
         -background  => 'white'
         )->pack(-side => 'top');

   $self->{'catab'}->{'top'}->Label(
         -text        => $parsed->{'FINGERPRINT'},
         -background  => 'white'
         )->pack(-side => 'top');

   $ca_left = $self->{'catab'}->{'top'}->Frame(
         -borderwidth => 50,
         -background  => 'white'
         )->pack(
           -fill   => 'both',
            -side   => 'left',
            -expand => 1
            );

   $ca_right = $self->{'catab'}->{'top'}->Frame(
         -borderwidth => 50,
         -background  => 'white'
         )->pack(
            -fill   => 'both',
            -side   => 'right',
            -expand => 1
            );

   if(defined($parsed->{'CN'})){
      _cert_line($ca_left, $ca_right, "Common Name:", 
            $parsed->{'CN'});
   }
   if(defined($parsed->{'EMAIL'})){
      _cert_line($ca_left, $ca_right, "eMail:", 
            $parsed->{'EMAIL'});
   }
   if(defined($parsed->{'O'})){
      _cert_line($ca_left, $ca_right, "Organization:", 
            $parsed->{'O'});
   }
   if(defined($parsed->{'OU'})){
      _cert_line($ca_left, $ca_right, "Organizational Unit:", 
            $parsed->{'OU'}->[0]);
   }
   if(defined($parsed->{'L'})){
      _cert_line($ca_left, $ca_right, "Location:", 
            $parsed->{'L'});
   }
   if(defined($parsed->{'ST'})){
      _cert_line($ca_left, $ca_right, "State:", 
            $parsed->{'ST'});
   }
   if(defined($parsed->{'C'})){
      _cert_line($ca_left, $ca_right, "Country:", 
            $parsed->{'C'});
   }
   _cert_line($ca_left, $ca_right, "", "");
   if(defined($parsed->{'NOTBEFORE'})){
      _cert_line($ca_left, $ca_right, "Creation Date:", 
            $parsed->{'NOTBEFORE'});
   }
   if(defined($parsed->{'NOTAFTER'})){
      _cert_line($ca_left, $ca_right, "Expiration Date:", 
            $parsed->{'NOTAFTER'});
   }
   
   ## frame for buttons
   if(not defined $self->{'catab'}->{'buttons'}) {
       $self->{'catab'}->{'buttons'} = $self->{'catab'}->Frame(
         -relief  => 'groove',
         )->pack(
            -side => 'bottom',
            -fill => 'x'
            );
    
       $self->{'catab'}->{'buttons'}->Button(
             -text    => 'View CA Details',
             -command => sub { $self->show_cert_details('CA') }
             )->pack(
                -side => 'left',
                -fill => 'both',
                -expand => 1
                );
    
       $self->{'catab'}->{'buttons'}->Button(
             -text    => 'Export CA Certificate',
             -command => sub { $self->{'CA'}->export_ca_cert($self) }
             )->pack(
                -side => 'left',
                -fill => 'both',
                -expand => 1
                );
    
       $self->{'catab'}->{'buttons'}->Button(
             -text    => 'Export CRL',
             -command => sub { $self->{'CA'}->export_crl($self) }
             )->pack(
                -side => 'right',
                -fill => 'both',
                -expand => 1
                );
   }

   ### notebooktab for certificate list
   if(not defined $self->{'certtab'}) {
      $self->{'certtab'} = $self->{'nb'}->add(
            'Certificates',
            -label     => 'Certificates',
            -underline => 1
            );
   }

   ## frame for title
   if(not defined $self->{'certtab'}->{'top'}) {
      $self->{'certtab'}->{'top'} = $self->{'certtab'}->Frame(
         -relief      => 'groove',
         -borderwidth => 2,
         )->pack(
            -side   => 'top',
            -expand => 0,
            -fill   => 'x'
            );

      $self->{'certtab'}->{'top'}->Label(
            -text => "Certificates"
            )->pack(-side => 'top');
   }

   ## frame for certificate list
   if(not defined $self->{'certtab'}->{'list'}) {
      $cert_list_frame = $self->{'certtab'}->Frame(
         -relief      => 'groove',
         -borderwidth => 2
         )->pack(
            -side   => 'top',
            -expand => 1,
            -fill   => 'both'
            );

      $self->{'certtab'}->{'list'} = $cert_list_frame->HList(
            -itemtype   => 'text',
            -selectmode => 'single',
            -columns    => 8,
            -header     => 1,
            -font       => 'small',
            -background => 'white'
            )->pack(
               -side => 'top',
               -expand => 1,
               -fill   => 'both'
               );

      _list_header($self->{'certtab'}->{'list'}, 0, 'Common Name');
      _list_header($self->{'certtab'}->{'list'}, 1, 'eMail');
      _list_header($self->{'certtab'}->{'list'}, 2, 'Organizational Unit');
      _list_header($self->{'certtab'}->{'list'}, 3, 'Organization');
      _list_header($self->{'certtab'}->{'list'}, 4, 'Location');
      _list_header($self->{'certtab'}->{'list'}, 5, 'State');
      _list_header($self->{'certtab'}->{'list'}, 6, 'Country');
      _list_header($self->{'certtab'}->{'list'}, 7, 'Status');

   }

   $self->{'CERT'}->read_certlist($self);
   
   $self->{'certtab'}->{'list'}->delete('all');

   my $i = 0;
   my $style_revoked = $self->{'certtab'}->{'list'}->ItemStyle(
         'text', 
         -foreground => '#800000',
	      -background => '#FFFFFF', 
         -selectforeground => '#FF0000'
         );
   my $style_valid = $self->{'certtab'}->{'list'}->ItemStyle(
         'text', 
         -foreground => '#008000',
	      -background => '#FFFFFF', 
         -selectforeground => '#00A000'
         );

   foreach(@{$self->{'CERT'}->{'certlist'}}) {
      $i++;
      (my $n, my $s) = split(/\%/, $_);
      my @line = split(/:/, $n);
      my $l = $self->{'certtab'}->{'list'}->add($i-1);
      for my $col(0 .. 6) {
         $self->{'certtab'}->{'list'}->itemCreate( $l, $col,
               -text => $line[$col]
               );
      }
      $self->{'certtab'}->{'list'}->itemCreate( $l, 7,
	    -style => ($s eq "VALID") ? $style_valid : $style_revoked,
            -text => $s
            );
   }

   # vertical scrollbar is only needed for more than 16 listitems
   if((@{$self->{'CERT'}->{'certlist'}} > 10) &&
      (not defined($self->{'certtab'}->{'list'}->{'scroll'}))) {
      $self->{'certtab'}->{'list'}->{'scroll'} = 
         $self->{'certtab'}->{'list'}->Scrollbar(
               -command => ['yview', 
                            $self->{'certtab'}->{'list'}
                           ]
               );
      $self->{'certtab'}->{'list'}->configure( 
            -yscrollcommand => ['set', 
                                $self->{'certtab'}->{'list'}->{'scroll'}
                               ]
            );

      $self->{'certtab'}->{'list'}->{'scroll'}->pack(
            -side => 'right', 
            -fill => 'y'
            );
   }

   # horizontal scrollbar
   if(not defined($self->{'certtab'}->{'list'}->{'hscroll'})) {
      $self->{'certtab'}->{'list'}->{'hscroll'} = 
         $self->{'certtab'}->{'list'}->Scrollbar(
               -command => ['xview', 
                            $self->{'certtab'}->{'list'}
                           ],
               -orient  => 'horizontal'
               );
      $self->{'certtab'}->{'list'}->configure( 
            -xscrollcommand => ['set', 
                                $self->{'certtab'}->{'list'}->{'hscroll'}
                               ]
            );

      $self->{'certtab'}->{'list'}->{'hscroll'}->pack(
            -side => 'bottom', 
            -fill => 'x'
            );
   }

   ## frame for buttons
   $state = @{$self->{'CERT'}->{'certlist'}}?'normal':'disabled';

   $self->{'certtab'}->{'buttons'}->destroy()
      if(defined $self->{'certtab'}->{'buttons'});

   $self->{'certtab'}->{'buttons'} = $self->{'certtab'}->Frame(
     -relief      => 'groove',
     -borderwidth => 2
     )->pack(
        -side   => 'bottom',
        -fill   => 'x'
        );

   $button = $self->{'certtab'}->{'buttons'}->Button(
         -text => 'Show Details',
         -command => sub { $self->show_cert_details() }
         )->pack(
            -side   => 'left',
            -fill   => 'x',
            -expand => 1, 
            );
   $button->configure( -state => $state);

   $button = $self->{'certtab'}->{'buttons'}->Button(
         -text => 'Show Extensions',
         -command => sub { $self->show_cert_exts() }
         )->pack(
            -side   => 'left',
            -fill   => 'x',
            -expand => 1
            );
   $button->configure( -state => $state);

   $button = $self->{'certtab'}->{'buttons'}->Button(
         -text => 'Export',
         -command => sub { $self->{'CERT'}->export_cert($self) }
         )->pack(
            -side => 'left',
            -fill => 'x',
            -expand => 1
            );
   $button->configure( -state => $state);

   $button = $self->{'certtab'}->{'buttons'}->Button(
         -text => 'Revoke',
         -command => sub { $self->{'CERT'}->revoke_cert($self) }
         )->pack(
            -side => 'left',
            -fill => 'x'
            );
   $button->configure( -state => $state);

   $button = $self->{'certtab'}->{'buttons'}->Button(
         -text => 'Delete',
         -command => sub { $self->{'CERT'}->del_cert($self) }
         )->pack(
            -side => 'left',
            -fill => 'x'
            );
   $button->configure( -state => $state);

   ### notebooktab for key lists (split list and buttons)
   if(not defined $self->{'keytab'}) {
      $self->{'keytab'} = $self->{'nb'}->add(
            'Keys',
            -label     => 'Keys',
            -underline => 0
            );
   }
   
   ## frame for title
   if(not defined $self->{'keytab'}->{'top'}) {
      $self->{'keytab'}->{'top'} = $self->{'keytab'}->Frame(
         -relief      => 'groove',
         -borderwidth => 2,
         )->pack(
            -side   => 'top',
            -expand => 0,
            -fill   => 'x'
            );

      $self->{'keytab'}->{'top'}->Label(
            -text => "Keys"
            )->pack(-side => 'top');
   } 
   
   ## frame for keylist
   if(not defined $self->{'keytab'}->{'list'}) {
      $key_list_frame = $self->{'keytab'}->Frame(
         -relief      => 'groove',
         -borderwidth => 2
         )->pack(
            -side   => 'top',
            -expand => 1,
            -fill   => 'both'
            );

      $self->{'keytab'}->{'list'} = $key_list_frame->HList(
            -itemtype   => 'text',
            -selectmode => 'single',
            -columns    => 7,
            -header     => 1,
            -font       => 'small',
            -background => 'white'
            )->pack(
               -side => 'top',
               -expand => 1,
               -fill   => 'both'
               );

      _list_header($self->{'keytab'}->{'list'}, 0, 'Common Name');
      _list_header($self->{'keytab'}->{'list'}, 1, 'eMail');
      _list_header($self->{'keytab'}->{'list'}, 2, 'Organizational Unit');
      _list_header($self->{'keytab'}->{'list'}, 3, 'Organization');
      _list_header($self->{'keytab'}->{'list'}, 4, 'Location');
      _list_header($self->{'keytab'}->{'list'}, 5, 'State');
      _list_header($self->{'keytab'}->{'list'}, 6, 'Country');

   }

   if((my $ret = $self->{'KEY'}->read_keylist($self)) == 1) {
      $self->{'keytab'}->{'list'}->delete('all');

      my $i = 0;
      foreach(@{$self->{'KEY'}->{'keylist'}}) {
         $i++;
         my @line = split(/:/, $_);
         my $l = $self->{'keytab'}->{'list'}->add($i-1);
         for my $col(0 .. 6) {
            $self->{'keytab'}->{'list'}->itemCreate( 
                  $l,
                  $col,
                  -text => $line[$col]
                  );
         }
      }
   }

   # vertical scrollbar
   if((@{$self->{'KEY'}->{'keylist'}} > 16) &&
      (not defined($self->{'keytab'}->{'list'}->{'scroll'}))) {
      $self->{'keytab'}->{'list'}->{'scroll'} = 
         $self->{'keytab'}->{'list'}->Scrollbar(
               -command => ['yview', 
                            $self->{'keytab'}->{'list'}
                           ]
               );
      $self->{'keytab'}->{'list'}->configure( 
            -yscrollcommand => ['set', 
                                $self->{'keytab'}->{'list'}->{'scroll'}
                               ]
            );

      $self->{'keytab'}->{'list'}->{'scroll'}->pack(
            -side => 'right', 
            -fill => 'y'
            );
   }

   # horizontal scrollbar
   if(not defined($self->{'keytab'}->{'list'}->{'hscroll'})) {
      $self->{'keytab'}->{'list'}->{'hscroll'} = 
         $self->{'keytab'}->{'list'}->Scrollbar(
               -command => ['xview', 
                            $self->{'keytab'}->{'list'}
                           ],
               -orient  => 'horizontal'
               );
      $self->{'keytab'}->{'list'}->configure( 
            -xscrollcommand => ['set', 
                                $self->{'keytab'}->{'list'}->{'hscroll'}
                               ]
            );

      $self->{'keytab'}->{'list'}->{'hscroll'}->pack(
            -side => 'bottom', 
            -fill => 'x'
            );
   }

   ## frame for buttons
   $state = @{$self->{'KEY'}->{'keylist'}}?'normal':'disabled';

   $self->{'keytab'}->{'buttons'}->destroy()
      if(defined $self->{'keytab'}->{'buttons'});

   $self->{'keytab'}->{'buttons'} = $self->{'keytab'}->Frame(
     -relief      => 'groove',
     -borderwidth => 2
     )->pack(
        -side   => 'bottom',
        -fill   => 'x'
        );
   
   $button = $self->{'keytab'}->{'buttons'}->Button(
         -text => 'Delete',
         -command => sub { $self->{'KEY'}->del_key($self) }
         )->pack(
            -side => 'left',
            -fill => 'x',
            -expand => 1
            );
   $button->configure( -state => $state);
   
   $button = $self->{'keytab'}->{'buttons'}->Button(
         -text => 'Export',
         -command => sub { $self->{'KEY'}->export_key($self) }
         )->pack(
            -side => 'left',
            -fill => 'x',
            -expand => 1
            );
   $button->configure( -state => $state);

   ### notebooktab for request lists (split list and buttons)
   if(not defined $self->{'reqtab'}) {
      $self->{'reqtab'} = $self->{'nb'}->add(
            'Requests',
            -label     => 'Requests',
            -underline => 0
            );
      
   }
   
   ## frame for title
   if(not defined $self->{'reqtab'}->{'top'}) {
      $self->{'reqtab'}->{'top'} = $self->{'reqtab'}->Frame(
         -relief      => 'groove',
         -borderwidth => 2,
         )->pack(
            -side   => 'top',
            -expand => 0,
            -fill   => 'x'
            );

      $self->{'reqtab'}->{'top'}->Label(
            -text => "Requests"
            )->pack(-side => 'top');
   }
   
   ## frame for requestlist
   if(not defined $self->{'reqtab'}->{'list'}) {
      $req_list_frame = $self->{'reqtab'}->Frame(
         -relief      => 'groove',
         -borderwidth => 2
         )->pack(
            -side   => 'top',
            -expand => 1,
            -fill   => 'both'
            );

      $self->{'reqtab'}->{'list'} = $req_list_frame->HList(
            -itemtype   => 'text',
            -selectmode => 'single',
            -columns    => 7,
            -header     => 1,
            -font       => 'small',
            -background => 'white'
            )->pack(
               -side => 'top',
               -expand => 1,
               -fill   => 'both'
               );

      _list_header($self->{'reqtab'}->{'list'}, 0, 'Common Name');
      _list_header($self->{'reqtab'}->{'list'}, 1, 'eMail');
      _list_header($self->{'reqtab'}->{'list'}, 2, 'Organizational Unit');
      _list_header($self->{'reqtab'}->{'list'}, 3, 'Organization');
      _list_header($self->{'reqtab'}->{'list'}, 4, 'Location');
      _list_header($self->{'reqtab'}->{'list'}, 5, 'State');
      _list_header($self->{'reqtab'}->{'list'}, 6, 'Country');

   }

   if((my $ret = $self->{'REQ'}->read_reqlist($self)) == 1) { 
      $self->{'reqtab'}->{'list'}->delete('all');

      my $i = 0;
      foreach(@{$self->{'REQ'}->{'reqlist'}}) {
         $i++;
         my @line = split(/:/, $_);
         my $l = $self->{'reqtab'}->{'list'}->add($i-1);
         for my $col(0 .. 6) {
            $self->{'reqtab'}->{'list'}->itemCreate(
                  $l,
                  $col,
                  -text => $line[$col]
                  );
         }
      }
   }

   # vertical scrollbar
   if((@{$self->{'REQ'}->{'reqlist'}} > 16) &&
      (not defined($self->{'reqtab'}->{'list'}->{'scroll'}))) {
      $self->{'reqtab'}->{'list'}->{'scroll'} = 
         $self->{'reqtab'}->{'list'}->Scrollbar(
               -command => ['yview', 
                            $self->{'reqtab'}->{'list'}
                           ]
               );
      $self->{'reqtab'}->{'list'}->configure( 
            -yscrollcommand => ['set', 
                                $self->{'reqtab'}->{'list'}->{'scroll'}
                               ]
            );

      $self->{'reqtab'}->{'list'}->{'scroll'}->pack(
            -side => 'right', 
            -fill => 'y'
            );
   } 

   # horizontal scrollbar
   if(not defined($self->{'reqtab'}->{'list'}->{'hscroll'})) {
      $self->{'reqtab'}->{'list'}->{'hscroll'} = 
         $self->{'reqtab'}->{'list'}->Scrollbar(
               -command => ['xview', 
                            $self->{'reqtab'}->{'list'}
                           ],
               -orient  => 'horizontal'
               );
      $self->{'reqtab'}->{'list'}->configure( 
            -xscrollcommand => ['set', 
                                $self->{'reqtab'}->{'list'}->{'hscroll'}
                               ]
            );

      $self->{'reqtab'}->{'list'}->{'hscroll'}->pack(
            -side => 'bottom', 
            -fill => 'x'
            );
   }

   ## frame for buttons
   $state = @{$self->{'REQ'}->{'reqlist'}}?'normal':'disabled';
   
   $self->{'reqtab'}->{'buttons'}->destroy() 
      if(defined($self->{'reqtab'}->{'buttons'}));

   $self->{'reqtab'}->{'buttons'} = $self->{'reqtab'}->Frame(
     -relief      => 'groove',
     -borderwidth => 2
     )->pack(
        -side   => 'bottom',
        -fill   => 'x'
        );
   
   $button = $self->{'reqtab'}->{'buttons'}->Button(
         -text => 'Show Details',
         -command => sub { $self->show_req_details() }
         )->pack(
            -side => 'left',
            -fill => 'x',
            -expand => 1
            );
   $button->configure( -state => $state);
   
   $button = $self->{'reqtab'}->{'buttons'}->Button(
         -text => 'New',
         -command => sub { $self->{'REQ'}->create_req($self) }
         )->pack(
            -side => 'left',
            -fill => 'x',
            -expand => 1
            );
   
   $button = $self->{'reqtab'}->{'buttons'}->Button(
         -text => 'Import',
         -command => sub { $self->{'REQ'}->import_req($self) }
         )->pack(
            -side => 'left',
            -fill => 'x',
            -expand => 1
            );
   
   $button = $self->{'reqtab'}->{'buttons'}->Button(
         -text => 'Delete',
         -command => sub { $self->{'REQ'}->del_req($self) }
         )->pack(
            -side => 'left',
            -fill => 'x',
            -expand => 1
            );
   $button->configure( -state => $state);
   
   $button = $self->{'reqtab'}->{'buttons'}->Button(
         -text => 'Sign (Server)',
         -command => sub { $self->{'REQ'}->sign_req($self, "server") }
         )->pack(
            -side => 'left',
            -fill => 'x',
            -expand => 1
            );
   $button->configure( -state => $state);
   
   $button = $self->{'reqtab'}->{'buttons'}->Button(
         -text => 'Sign (Client)',
         -command => sub { $self->{'REQ'}->sign_req($self, "client") }
         )->pack(
            -side => 'left',
            -fill => 'x',
            -expand => 1
            );
   $button->configure( -state => $state);

   ## done -> pack notebook
   $self->{'nb'}->pack(
         -expand => 1,
         -fill => "both",
         -padx => 5,
         -pady => 5,
         -side => "top"
         );

   return;
}

### actualize menu
sub act_menu {
   my $self = shift;

   my $openstate   = 'disabled';
   my $delstate    = 'disabled';
   my $configstate = 'disabled';

   if(@{$self->{'CA'}->{'calist'}} > 0) {
      $openstate = $delstate = 'normal';
   }
   if(defined($self->{'CA'}->{'actca'})) {
      $configstate = 'normal';
   }

   $self->{'menu'}->{'menuca'}->{'open'}->configure ( 
      -state => $openstate
   );
   $self->{'menu'}->{'menuca'}->{'del'}->configure ( 
      -state => $delstate
   );
   $self->{'menu'}->{'menuconfig'}->configure ( 
      -state => $configstate
   );
}

### create the menubar
sub create_menu {
   my $self = shift;

   my $menu = $self->{'mw'}->Frame(
         -relief      => 'groove',
         -borderwidth => 2
         )->pack(
            -side => 'top',
            -fill => 'x'
            );

   # menu for CA
   my $menu_ca = $menu->Menubutton(
         -text         => 'CA',
         -underline    => 0
         )->pack(-side => 'left');
      
   my $menu_open = $menu_ca->command(
         -label   => 'Open CA',
         -command => sub{ $self->{'CA'}->open_ca($self) }
         );
      $menu_open->configure(-state => 'disabled');

   my $menu_create = $menu_ca->command(
         -label   => 'New CA',
         -command => sub{ $self->{'CA'}->create_ca($self) }
         );

   my $menu_del = $menu_ca->command(
         -label   => 'Delete CA',
         -command => sub{ $self->{'CA'}->delete_ca($self) }
         );
      $menu_del->configure(-state => 'disabled');
      
      $menu_ca->separator();

   my $menu_exit = $menu_ca->command(
         -label   => 'Exit',
         -command => sub{ $self->exit_clean() }
         );

   $self->{'menu'} = $menu;
   $self->{'menu'}->{'menuca'} = $menu_ca;
   $self->{'menu'}->{'menuca'}->{'open'}   = $menu_open;
   $self->{'menu'}->{'menuca'}->{'create'} = $menu_create;
   $self->{'menu'}->{'menuca'}->{'del'}    = $menu_del;
   $self->{'menu'}->{'menuca'}->{'exit'}   = $menu_exit;

   # menu for configuration
   my $menu_config = $menu->Menubutton(
         -text      => 'Configuration',
         -underline => 1
         )->pack(-side => 'left');
      $menu_config->configure(-state => 'disabled');

   $menu_config->command(
         -label => 'Experts Only!!'
         );

   $menu_config->separator();

   $menu_config->command(
         -label   => 'Server Certificates',
         -command => sub{ $self->{'CONFIG'}->config_server($self) }
         );

   $menu_config->command(
         -label   => 'Client Certificates',
         -command => sub{ $self->{'CONFIG'}->config_client($self) }
         );

   $menu_config->command(
         -label   => 'Revocation Lists',
         -command => sub{ $self->{'CONFIG'}->config_crl($self) }
         );

   $self->{'menu'}->{'menuconfig'} = $menu_config;


   # menu for help :-))
   my $menu_help = $menu->Menubutton(
         -text => 'Help'
         )->pack(-side => 'right');

      $menu_help->command(
            -label   => 'Help',
            -command => sub{ $self->show_help() }
            );

      $menu_help->command(
            -label   => 'About',
            -command => sub{ $self->about() }
            );

   return;
}

sub exit_clean {
   my $self = shift;
   my $ret  = shift;

   $ret = 0 unless(defined $ret);
   
   $self->{'mw'}->destroy;

   exit($ret);
}

## pop-up to display request/cert as TXT
sub show_text {
   my $self  = shift;
   my $type  = shift;
   my $getca = shift;
   my $ca = $self->{'CA'}->{'actca'};

   my($data, $dataname, $dialog, $txt, $tmp, $parsed, $ind, $list, $tab);
   
   if($type eq 'CERT') {
      $list = 'certlist';
      $tab  = 'certtab';
   } elsif($type eq 'REQ') {
      $list = 'reqlist';
      $tab  = 'reqtab';
   } else {
      $self->print_warning("Invalid data type: $type");
      return;
   }

   $ind = $self->{$tab}->{'list'}->info('selection');

   if((not defined $ind) && ($getca ne 'CA')) { 
      $self->print_warning("Nothing selected");
      return;
   } 

   if((defined($getca)) && ($getca eq 'CA')) {
      $parsed = $self->{$type}->parse_cert($self, 'CA');
   } elsif($type eq 'CERT') {
      ($data, $tmp) = split(/\%/, $self->{$type}->{$list}->[$ind]);
      $dataname = MIME::Base64::encode($data, '');
      $parsed = $self->{$type}->parse_cert($self, $dataname);
   } elsif($type eq 'REQ') {
      $data = $self->{$type}->{$list}->[$ind];
      $dataname = MIME::Base64::encode($data, '');
      $parsed = $self->{$type}->parse_req($self, $dataname);
   }

   defined($parsed) || $self->print_error("Can't read Data");

   $dialog = $self->{'mw'}->Toplevel();
   $dialog->title("Tiny CA Management $self->{'version'} - Text");

   $dialog->Label(
         -text => ($type eq 'CERT')?'Certificate':'Request',
         -font => 'big'
         )->pack(
            -fill => 'y',
            -side => 'top'
            );

   $tmp = $dialog->Frame(
         -borderwidth => 3,
         -relief      => 'groove',
         )->pack(
            -fill => 'x',
            -side => 'top',
            -expand => 1
            );
   $txt = $tmp->Scrolled('Text')->pack();
   $txt->insert('end', $parsed->{'TEXT'});
   $txt->configure( 
         -background => 'white',
         -state      => 'disabled',
         -takefocus  => 0
         );

   $tmp = $dialog->Frame(
         -borderwidth => 3
         )->pack(
            -fill => 'both',
            -expand => 1,
            -side => 'bottom'
            );

   $dialog->Button(
         -text => 'OK',
         -command => sub { $dialog->destroy() }
         )->pack(
            -side   => 'left',
            -fill   => 'x',
            -expand => 1
            );
}

## pop-up to display request informations
sub show_req_details {
   my $self = shift;
   my $ca = $self->{'CA'}->{'actca'};
   my $ind = $self->{'reqtab'}->{'list'}->info('selection');

   my($req, $reqname, $dialog, $parsed, $tmp, $tmpl, $tmpr);

   if(not defined $ind) { 
      $self->print_info("Please select a request first");
      return;
   }

   $req = $self->{'REQ'}->{'reqlist'}->[$ind];

   $reqname = MIME::Base64::encode($req, '');

   $parsed = $self->{'REQ'}->parse_req($self, $reqname);
   
   defined($parsed) || $self->print_error("Can't read Request");

   $dialog = $self->{'mw'}->Toplevel();
   $dialog->title("Tiny CA Management $self->{'version'} - Request Details");

   $dialog->Label(
         -text => 'Request Details',
         -font => 'big'
         )->pack(
            -fill => 'y',
            -side => 'top'
            );

   $tmp = $dialog->Frame(
         -borderwidth => 3,
         -relief      => 'groove',
         )->pack(
            -fill => 'x',
            -side => 'top',
            -expand => 1
            );

   $tmpl = $tmp->Frame( 
         -borderwidth => 15,
         -background  => 'white'
         )->pack(
            -fill   => 'both',
            -expand => 1,
            -side   => 'left'
            );

   $tmpr = $tmp->Frame( 
         -borderwidth => 15,
         -background  => 'white'
         )->pack(
            -fill   => 'both',
            -expand => 1,
            -side   => 'right'
            );

   if(defined($parsed->{'CN'})){    
      _cert_line($tmpl, $tmpr, "Common Name:", 
            $parsed->{'CN'});
   }
   if(defined($parsed->{'EMAIL'})){    
      _cert_line($tmpl, $tmpr, "eMail:", 
            $parsed->{'EMAIL'});
   }
   if(defined($parsed->{'S'})){    
      _cert_line($tmpl, $tmpr, "Surname:", 
            $parsed->{'S'});
   }
   if(defined($parsed->{'G'})){    
      _cert_line($tmpl, $tmpr, "Given Name:", 
            $parsed->{'G'});
   }
   if(defined($parsed->{'N'})){    
      _cert_line($tmpl, $tmpr, "Name:", 
            $parsed->{'N'});
   }
   if(defined($parsed->{'T'})){    
      _cert_line($tmpl, $tmpr, "Title:", 
            $parsed->{'T'});
   }
   if(defined($parsed->{'I'})){    
      _cert_line($tmpl, $tmpr, "Initials:", 
            $parsed->{'I'});
   }
   if(defined($parsed->{'D'})){    
      _cert_line($tmpl, $tmpr, "Description:", 
            $parsed->{'D'});
   }
   if(defined($parsed->{'O'})){    
      _cert_line($tmpl, $tmpr, "Organization:", 
            $parsed->{'O'});
   }
   if(defined($parsed->{'OU'})){    
      _cert_line($tmpl, $tmpr, "Organizational Unit:", 
            $parsed->{'OU'}->[0]);
   }
   if(defined($parsed->{'L'})){    
      _cert_line($tmpl, $tmpr, "Location:", 
            $parsed->{'L'});
   }
   if(defined($parsed->{'ST'})){    
      _cert_line($tmpl, $tmpr, "State:", 
            $parsed->{'ST'});
   }
   if(defined($parsed->{'C'})){    
      _cert_line($tmpl, $tmpr, "Country:", 
            $parsed->{'C'});
   }
   _cert_line($tmpl, $tmpr, "", "");
   _cert_line($tmpl, $tmpr, "", "");
   if(defined($parsed->{'PK_ALGORITHM'})){    
      _cert_line($tmpl, $tmpr, "Public Key Algorithm:", 
            $parsed->{'PK_ALGORITHM'});
   }
   if(defined($parsed->{'KEYSIZE'})){    
      _cert_line($tmpl, $tmpr, "Keylength:", 
            $parsed->{'KEYSIZE'});
   }
   if(defined($parsed->{'TYPE'})){    
      _cert_line($tmpl, $tmpr, "Type:", $parsed->{'TYPE'});
   }

    $tmp = $dialog->Frame(
          -borderwidth => 3
          )->pack(
             -fill => 'both',
             -expand => 1,
             -side => 'bottom'
             );

   $dialog->Button(
         -text    => 'OK',
         -command => sub { $dialog->destroy() }
         )->pack(
            -side   => 'left',
            -fill   => 'x',
            -expand => 1
            );

   $dialog->Button(
         -text => 'Show Extensions',
         -command => sub { $self->show_req_exts() }
         )->pack(
            -side   => 'left',
            -fill   => 'x',
            -expand => 1
            );

   $dialog->Button(
         -text => 'View Request',
         -command => sub { $self->show_text('REQ') }
         )->pack(
            -side   => 'right',
            -fill   => 'x',
            -expand => 1
            );
}

## pop-up for request extensions
sub show_req_exts {
   my $self = shift;
   my $ca   = $self->{'CA'}->{'actca'};
   my $ind  = $self->{'reqtab'}->{'list'}->info('selection');

   my($req, $status, $reqname, $dialog, $parsed, $tmp, $tmpl, 
      $tmpr, $key, $val, $row);

   if(not defined $ind) { 
      $self->print_info("Please select a request first");
      return;
   }

   ($req, $status) = split(/\%/, $self->{'REQ'}->{'reqlist'}->[$ind]);

   $reqname = MIME::Base64::encode($req, '');

   $parsed = $self->{'REQ'}->parse_req( $self, $reqname);

   defined($parsed) || $self->print_error("can't read request");

   if(not keys(%{$parsed->{'EXT'}})) {
      $self->print_warning("no extensions found");
      return;
   }

   $dialog = $self->{'mw'}->Toplevel();
   $dialog->title(
         "Tiny CA Management $self->{'version'} - Request Extensions"
         );

   $dialog->Label(
         -text => 'Requested X.509v3 Extensions',
         -font => 'big'
         )->pack(
            -fill => 'x',
            -side => 'top'
            );

   $tmp = $dialog->Frame(
         -borderwidth => 3,
         -relief      => 'groove',
         )->pack(
            -fill => 'both',
            -side => 'top',
            -expand => 1
            );

   while(($key, $val) = each(%{$parsed->{'EXT'}})) {
      $row = $dialog->Frame(
            -borderwidth => 1,
            -relief      => 'groove',
            -background  => 'white'
            )->pack(
               -fill => 'both',
               -side => 'top',
               -expand => 1
               );

      $tmp = $row->Frame(
            -borderwidth => 0,
            -background  => 'white'
            )->pack(
               -fill => 'both',
               -side => 'top',
               -expand => 1
               );

      $tmpl = $tmp->Frame(
            -borderwidth => 5,
            -background  => 'white'
            )->pack(
               -fill   => 'both',
               -expand => 1,
               -side   => 'left'
               );
   
      $tmpr = $tmp->Frame(
            -borderwidth => 5,
            -background  => 'white'
            )->pack(
               -fill   => 'both',
               -expand => 1,
               -side   => 'right'
               );

      $tmpl->Label(
            -text       => "$key:",
            -background => 'white',
            -font       => 'small'
            )->pack(
               -fill   => 'y',
               -side   => 'left',
               -anchor => 'nw'
               );

      $tmpr->Label(
            -text       => "$val->[0]",
            -background => 'white',
            -font       => 'small'
            )->pack(
               -fill   => 'y',
               -side   => 'right',
               -anchor => 'nw'
               );
            
      if(@{$val} > 1) {
         for(my $i = 1; $val->[$i]; $i++) {
         $tmp = $row->Frame(
               -borderwidth => 0,
               -relief      => 'groove',
               -background  => 'white'
               )->pack(
                  -fill => 'both',
                  -side => 'top',
                  -expand => 1
                  );
   
         $tmpr = $tmp->Frame(
               -borderwidth => 5,
               -background  => 'white'
               )->pack(
                  -fill   => 'both',
                  -expand => 1,
                  -side   => 'right'
                  );

         $tmpr->Label( 
               -text       => $val->[$i],
               -background => 'white',
               -font       => 'small'
               )->pack(
                  -fill   => 'y',
                  -side   => 'top',
                  -anchor => 'ne'
                  );
         }
      }

   }

    $tmp = $dialog->Frame(
          -borderwidth => 3
          )->pack(
             -fill => 'both',
             -expand => 1,
             -side => 'bottom'
             );

   $dialog->Button(
         -text => 'OK',
         -command => sub { $dialog->destroy() }
         )->pack(
            -side   => 'left',
            -fill   => 'x',
            -expand => 1
            );

   $dialog->Button(
         -text => 'View Request',
         -command => sub { $self->show_text('REQ') }
         )->pack(
            -side   => 'left',
            -fill   => 'x',
            -expand => 1
            );
}

## pop-up for certificate extensions
sub show_cert_exts {
   my $self  = shift;
   my $getca = shift;
   my $ca    = $self->{'CA'}->{'actca'};
   my $ind   = $self->{'certtab'}->{'list'}->info('selection');

   my($cert, $status, $certname, $dialog, $parsed, $tmp, $tmpl, 
      $tmpr, $key, $val, $row);

   if((not defined $ind) && ((not defined $getca) || ($getca ne 'CA'))) { 
      $self->print_info("Please select a certificate first");
      return;
   }

   if((defined($getca)) && ($getca eq 'CA')) {
      $parsed = $self->{'CERT'}->parse_cert($self, 'CA');
   } else {
      ($cert, $status) = split(/\%/, $self->{'CERT'}->{'certlist'}->[$ind]);

      $certname = MIME::Base64::encode($cert, '');

      $parsed = $self->{'CERT'}->parse_cert( $self, $certname);
   }

   defined($parsed) || $self->print_error("can't read Certificate");

   keys(%{$parsed->{'EXT'}}) || return;

   $dialog = $self->{'mw'}->Toplevel();
   $dialog->title(
         "Tiny CA Management $self->{'version'} - Certificate Extensions"
         );

   $dialog->Label(
         -text => 'Certificate X.509v3 Extensions',
         -font => 'big'
         )->pack(
            -fill => 'x',
            -side => 'top'
            );

   $tmp = $dialog->Frame(
         -borderwidth => 3,
         -relief      => 'groove',
         )->pack(
            -fill => 'both',
            -side => 'top',
            -expand => 1
            );

   while(($key, $val) = each(%{$parsed->{'EXT'}})) {
      $row = $dialog->Frame(
            -borderwidth => 1,
            -relief      => 'groove',
            -background  => 'white'
            )->pack(
               -fill => 'both',
               -side => 'top',
               -expand => 1
               );

      $tmp = $row->Frame(
            -borderwidth => 0,
            -background  => 'white'
            )->pack(
               -fill => 'both',
               -side => 'top',
               -expand => 1
               );

      $tmpl = $tmp->Frame(
            -borderwidth => 5,
            -background  => 'white'
            )->pack(
               -fill   => 'both',
               -expand => 1,
               -side   => 'left'
               );
   
      $tmpr = $tmp->Frame(
            -borderwidth => 5,
            -background  => 'white'
            )->pack(
               -fill   => 'both',
               -expand => 1,
               -side   => 'right'
               );

      $tmpl->Label(
            -text       => "$key:",
            -background => 'white',
            -font       => 'small'
            )->pack(
               -fill   => 'y',
               -side   => 'left',
               -anchor => 'nw'
               );

      $tmpr->Label(
            -text       => "$val->[0]",
            -background => 'white',
            -font       => 'small'
            )->pack(
               -fill   => 'y',
               -side   => 'right',
               -anchor => 'nw'
               );
            
      if(@{$val} > 1) {
         for(my $i = 1; $val->[$i]; $i++) {
         $tmp = $row->Frame(
               -borderwidth => 0,
               -relief      => 'groove',
               -background  => 'white'
               )->pack(
                  -fill => 'both',
                  -side => 'top',
                  -expand => 1
                  );
   
         $tmpr = $tmp->Frame(
               -borderwidth => 5,
               -background  => 'white'
               )->pack(
                  -fill   => 'both',
                  -expand => 1,
                  -side   => 'right'
                  );

         $tmpr->Label( 
               -text       => $val->[$i],
               -background => 'white',
               -font       => 'small'
               )->pack(
                  -fill   => 'y',
                  -side   => 'top',
                  -anchor => 'ne'
                  );
         }
      }

   }

    $tmp = $dialog->Frame(
          -borderwidth => 3
          )->pack(
             -fill => 'both',
             -expand => 1,
             -side => 'bottom'
             );

   $dialog->Button(
         -text => 'OK',
         -command => sub { $dialog->destroy() }
         )->pack(
            -side   => 'left',
            -fill   => 'x',
            -expand => 1
            );

   $dialog->Button(
         -text => 'View Certificate',
         -command => sub { $self->show_text('CERT', $getca) }
         )->pack(
            -side   => 'left',
            -fill   => 'x',
            -expand => 1
            );
}

## pop-up for certificate information 
sub show_cert_details {
   my $self  = shift;
   my $getca = shift;

   my $ca    = $self->{'CA'}->{'actca'};
   my $ind   = $self->{'certtab'}->{'list'}->info('selection');

   my($cert, $status, $certname, $dialog, $parsed, $tmp, $tmpl, 
      $tmpr, $key, $val);

   if((not defined $ind) && ((not defined $getca) || ($getca ne 'CA'))) { 
      $self->print_info("Please select a certificate first");
      return;
   }

   if((defined($getca)) && ($getca eq 'CA')) {
      $parsed = $self->{'CERT'}->parse_cert($self, 'CA');
   } else {
      ($cert, $status) = split(/\%/, $self->{'CERT'}->{'certlist'}->[$ind]);

      $certname = MIME::Base64::encode($cert, '');

      $parsed = $self->{'CERT'}->parse_cert(
               $self, 
               $certname
               );
   }

   defined($parsed) || $self->print_error("can't read Certificate");

   $dialog = $self->{'mw'}->Toplevel();
   $dialog->title(
         "Tiny CA Management $self->{'version'} - Certificate Details"
         );

   $dialog->Label(
         -text => 'Certificate Details',
         -font => 'big'
         )->pack(
            -fill => 'x',
            -side => 'top'
            );

   $tmp = $dialog->Frame(
         -borderwidth => 3,
         -relief      => 'groove'
         )->pack(
            -fill => 'both',
            -side => 'top',
            -expand => 1
            );

   $tmp->Label(
         -text => $parsed->{'FINGERPRINT'},
         -background  => 'white'
         )->pack(
            -fill => 'both',
            -side => 'top'
            );

   $tmpl = $tmp->Frame( 
         -borderwidth => 15,
         -background  => 'white'
         )->pack(
            -fill   => 'both',
            -expand => 1,
            -side   => 'left'
            );

   $tmpr = $tmp->Frame( 
         -borderwidth => 15,
         -background  => 'white'
         )->pack(
            -fill   => 'both',
            -expand => 1,
            -side   => 'right'
            );

   unless(defined($getca) && $getca ne 'CA') {
      _cert_line($tmpl, $tmpr, "Status:", $status);
   }
   if(defined($parsed->{'SERIAL'})){    
      _cert_line($tmpl, $tmpr, "Serial:", 
            $parsed->{'SERIAL'});
   }
      _cert_line($tmpl, $tmpr, "", ""); 
   if(defined($parsed->{'CN'})){    
      _cert_line($tmpl, $tmpr, "Common Name:", 
            $parsed->{'CN'});
   }
   if(defined($parsed->{'EMAIL'})){    
      _cert_line($tmpl, $tmpr, "eMail:", 
            $parsed->{'EMAIL'});
   }
   if(defined($parsed->{'S'})){    
      _cert_line($tmpl, $tmpr, "Surname:", 
            $parsed->{'S'});
   }
   if(defined($parsed->{'G'})){    
      _cert_line($tmpl, $tmpr, "Given Name:", 
            $parsed->{'G'});
   }
   if(defined($parsed->{'N'})){    
      _cert_line($tmpl, $tmpr, "Name:", 
            $parsed->{'N'});
   }
   if(defined($parsed->{'T'})){    
      _cert_line($tmpl, $tmpr, "Title:", 
            $parsed->{'T'});
   }
   if(defined($parsed->{'I'})){    
      _cert_line($tmpl, $tmpr, "Initials:", 
            $parsed->{'I'});
   }
   if(defined($parsed->{'D'})){    
      _cert_line($tmpl, $tmpr, "Description:", 
            $parsed->{'D'});
   }
   if(defined($parsed->{'O'})){    
      _cert_line($tmpl, $tmpr, "Organization:", 
            $parsed->{'O'});
   }
   if(defined($parsed->{'OU'}->[0])){    
      _cert_line($tmpl, $tmpr, "Organizational Unit:", 
            $parsed->{'OU'}->[0]);
   }
   if(defined($parsed->{'L'})){    
      _cert_line($tmpl, $tmpr, "Location:", 
            $parsed->{'L'});
   }
   if(defined($parsed->{'ST'})){    
      _cert_line($tmpl, $tmpr, "State:", 
            $parsed->{'ST'});
   }
   if(defined($parsed->{'C'})){    
      _cert_line($tmpl, $tmpr, "Country:", 
            $parsed->{'C'});
   }
      _cert_line($tmpl, $tmpr, "", "");
   if(defined($parsed->{'NOTBEFORE'})){    
      _cert_line($tmpl, $tmpr, "Creation Date:", 
            $parsed->{'NOTBEFORE'});
   }
   if(defined($parsed->{'NOTAFTER'})){    
      _cert_line($tmpl, $tmpr, "Expiration Date:", 
            $parsed->{'NOTAFTER'});
   }
      _cert_line($tmpl, $tmpr, "", "");
   if(defined($parsed->{'PK_ALGORITHM'})){    
      _cert_line($tmpl, $tmpr, "Public Key Algorithm:", 
            $parsed->{'PK_ALGORITHM'});
   }
   if(defined($parsed->{'KEYSIZE'})){    
      _cert_line($tmpl, $tmpr, "Keylength:", 
            $parsed->{'KEYSIZE'});
   }
   if(defined($parsed->{'SIG_ALGORITHM'})){    
      _cert_line($tmpl, $tmpr, "Signature Algorithm:", 
            $parsed->{'SIG_ALGORITHM'});
   }

   $dialog->Button(
         -text => 'OK',
         -command => sub { $dialog->destroy() }
         )->pack(
            -side   => 'left',
            -fill   => 'x',
            -expand => 1
            );

   $dialog->Button(
         -text => 'Show Extensions',
         -command => sub { $self->show_cert_exts($getca) }
         )->pack(
            -side   => 'left',
            -fill   => 'x',
            -expand => 1
            );

   $dialog->Button(
         -text => 'View Certificate',
         -command => sub { $self->show_text('CERT', $getca) }
         )->pack(
            -side   => 'left',
            -fill   => 'x',
            -expand => 1
            );
}

## pop-up to verify request import
sub show_req_import_verification {
   my $self   = shift;
   my $parsed = shift;

   my($req, $reqname, $dialog, $tmp, $tmpl, $tmpr);

   $dialog = $self->{'mw'}->DialogBox( 
         -title          => 'Import Request',
         -buttons        => ['Import', 'Cancel'],
         -default_button => 'Import'
         );

   $dialog->Label(
         -text => 'Request Details',
         -font => 'big'
         )->pack(
            -fill => 'y',
            -side => 'top'
            );

   $tmp = $dialog->Frame(
         -borderwidth => 3,
         -relief      => 'groove',
         )->pack(
            -fill => 'x',
            -side => 'top',
            -expand => 1
            );

   $tmpl = $tmp->Frame( 
         -borderwidth => 15,
         -background  => 'white'
         )->pack(
            -fill   => 'both',
            -expand => 1,
            -side   => 'left'
            );

   $tmpr = $tmp->Frame( 
         -borderwidth => 15,
         -background  => 'white'
         )->pack(
            -fill   => 'both',
            -expand => 1,
            -side   => 'right'
            );

   if(defined $parsed->{'CN'}) {
      _cert_line($tmpl, $tmpr, "Common Name:", 
            $parsed->{'CN'});
   }
   if(defined $parsed->{'EMAIL'}) {
      _cert_line($tmpl, $tmpr, "eMail:", 
            $parsed->{'EMAIL'});
   }
   if(defined($parsed->{'S'})){    
      _cert_line($tmpl, $tmpr, "Surname:", 
            $parsed->{'S'});
   }
   if(defined($parsed->{'G'})){    
      _cert_line($tmpl, $tmpr, "Given Name:", 
            $parsed->{'G'});
   }
   if(defined($parsed->{'N'})){    
      _cert_line($tmpl, $tmpr, "Name:", 
            $parsed->{'N'});
   }
   if(defined($parsed->{'T'})){    
      _cert_line($tmpl, $tmpr, "Title:", 
            $parsed->{'T'});
   }
   if(defined($parsed->{'I'})){    
      _cert_line($tmpl, $tmpr, "Initials:", 
            $parsed->{'I'});
   }
   if(defined($parsed->{'D'})){    
      _cert_line($tmpl, $tmpr, "Description:", 
            $parsed->{'D'});
   }
   if(defined $parsed->{'O'}) {
      _cert_line($tmpl, $tmpr, "Organization:", 
            $parsed->{'O'});
   }
   if(defined $parsed->{'OU'}) { 
      _cert_line($tmpl, $tmpr, "Organizational Unit:", 
            $parsed->{'OU'}->[0]);
   }
   if(defined $parsed->{'L'}) {
      _cert_line($tmpl, $tmpr, "Location:", 
            $parsed->{'L'});
   }
   if(defined $parsed->{'ST'}) {
      _cert_line($tmpl, $tmpr, "State:", 
            $parsed->{'ST'});
   }
   if(defined $parsed->{'C'}) {
      _cert_line($tmpl, $tmpr, "Country:", 
            $parsed->{'C'});
   }
   _cert_line($tmpl, $tmpr, "", "");
   _cert_line($tmpl, $tmpr, "", "");
   if(defined $parsed->{'PK_ALGORITHM'}) {
      _cert_line($tmpl, $tmpr, "Public Key Algorithm:", 
         $parsed->{'PK_ALGORITHM'});
   }
   if(defined $parsed->{'KEYSIZE'}) {
      _cert_line($tmpl, $tmpr, "Keylength:", 
            $parsed->{'KEYSIZE'});
   }
   if(defined $parsed->{'TYPE'}) {
      _cert_line($tmpl, $tmpr, "Type:", 
            $parsed->{'TYPE'});
   }

   $dialog->Show(1);
}

## get name for deleting a CA
sub show_del_dialog {
   my $self = shift;
   my $ind  = shift;

   my ($labelframe, $listframe, $box, $scroll, $dialog);

   $dialog = $self->{'mw'}->DialogBox(
         -title          => 'Delete CA', 
         -buttons        => ['Delete', 'Cancel'], 
         -default_button => 'Delete'
         );
   $dialog->geometry('30x13');

   $labelframe = $dialog->add( 
         'Frame',
         -borderwidth => 10
         )->pack(-fill => 'x');

   $listframe = $dialog->add(
         'Frame',
         -borderwidth => 3
         )->pack(
            -fill   => 'both',
            -expand => 1
            );

   $labelframe->Label(
         -text => 'Delete CA'
         )->pack(-side => 'top');

   $box = $listframe->Listbox(
         -relief     => 'sunken',
         -setgrid    => 1,
         -background => 'white',
         );
   
   foreach(@{$self->{'CA'}->{'calist'}}) {
      next if (not defined $_ );
      $box->insert('end', $_);
   }

   $scroll = $box->Scrollbar(-command => ['yview', $box]);
   $box->configure(-yscrollcommand => ['set', $scroll]);

   $box->pack(
         -side   => 'top', 
         -fill   => 'both', 
         -expand => 1
         );

   $scroll->pack(
         -side => 'right', 
         -fill => 'y'
         );

   return undef if($dialog->Show(1) eq 'Cancel');

   $ind = $box->curselection;
}

## get name for opening a CA
sub show_open_dialog {
   my $self = shift;
   my $ind  = shift;

   my ($dialog, $labelframe, $listframe, $box, $scroll);

   $dialog = $self->{'mw'}->DialogBox(
         -title          => 'Open CA', 
         -buttons        => ['Open', 'Cancel'], 
         -default_button => 'Open'
         );
   $dialog->geometry('30x13');

   $labelframe = $dialog->add(
         'Frame',
         -borderwidth => 10
         )->pack(-fill => 'x');

   $listframe = $dialog->add(
         'Frame',
         -borderwidth => 3
         )->pack(
            -fill   => 'both',
            -expand => 1
            );

   $labelframe->Label(
         -text => 'Open CA'
         )->pack(-side => 'top');

   $box = $listframe->Listbox(
         -relief     => 'sunken',
         -setgrid    => 1,
         -background => 'white'
         );
   
   foreach(@{$self->{'CA'}->{'calist'}}) {
      next if (not defined $_ );
      $box->insert('end', $_);
   }

   $scroll = $box->Scrollbar(-command => ['yview', $box]);
   $box->configure(-yscrollcommand => ['set', $scroll]);
   
   $box->pack(
         -side   => 'top', 
         -fill   => 'both', 
         -expand => 1
         );
   
   $scroll->pack(
         -side => 'right', 
         -fill => 'y'
         );

   return undef if($dialog->Show(1) eq 'Cancel');

   $ind = $box->curselection;
}

## get data for creating a new request
sub show_req_dialog {
   my $self = shift;
   my $opt  = shift;

   my ($inputframe, $labelframe, $dialog, $e);

   $dialog = $self->{'mw'}->DialogBox( 
         -title          => 'Create Request',
         -buttons        => ['Create', 'Cancel'],
         -default_button => 'Create'
         );

   $labelframe = $dialog->Frame( 
         -borderwidth => 10
         )->pack(-fill => 'x');

   $labelframe->Label(
         -text => 'Create a new Request',
         -font => 'big'
         )->pack(-side => 'top');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'Common Name (eg, your Name,'
         )->pack(-side => 'left');
   my $ent = $inputframe->Entry(
         -textvariable => \$opt->{'CN'}
         )->pack(-side => 'right');
   $ent->focus();
   $ent->icursor(0);

   $inputframe = $dialog->Frame( 
         -borderwidth => 3
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => '                        your eMail Address'
         )->pack(-side => 'left');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => '                        or the Servers Name)'
         )->pack(-side => 'left');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'eMail Address:'
         )->pack(-side => 'left');
   $inputframe->Entry(
         -textvariable => \$opt->{'EMAIL'}
         )->pack(-side => 'right');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'Password (protect your private Key):'
         )->pack(-side => 'left');
   $inputframe->Entry(
         -show         => '*',
         -textvariable => \$opt->{'passwd'}
         )->pack(-side => 'right');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'Password (confirmation):'
         )->pack(-side => 'left');
   $inputframe->Entry(
         -show         => '*',
         -textvariable => \$opt->{'passwd2'}
         )->pack(-side => 'right');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'Country Name (2 letter code):'
         )->pack(-side => 'left');
   $e = $inputframe->Entry(
         -textvariable => \$opt->{'C'}
         )->pack(-side => 'right');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'State or Province Name:'
         )->pack(-side => 'left');
   $e = $inputframe->Entry( 
         -textvariable => \$opt->{'ST'}
         )->pack(-side => 'right');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'Locality Name (eg, city):'
         )->pack(-side => 'left');
   $e = $inputframe->Entry(
         -textvariable => \$opt->{'L'}
         )->pack(-side => 'right');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'Organization Name (eg, company):'
         )->pack(-side => 'left');
   $e = $inputframe->Entry(
         -textvariable => \$opt->{'O'}
         )->pack(-side => 'right');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'Organizational Unit Name (eg, section):'
         )->pack(-side => 'left');
   $e = $inputframe->Entry(
         -textvariable => \$opt->{'OU'}
         )->pack(-side => 'right');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'Keylength'
         )->pack(-side => 'left');
   $inputframe->Radiobutton( 
         -text  => '1024',
         -value => 1024,
         -variable => \$opt->{'bits'}
         )->pack(-side => 'right');
   $inputframe->Radiobutton(
         -text  => '2048',
         -value => 2048,
         -variable => \$opt->{'bits'}
         )->pack(-side => 'right');
   $inputframe->Radiobutton(
         -text  => '4096',
         -value => 4096,
         -variable => \$opt->{'bits'}
         )->pack(-side => 'right');


   $dialog->Show(1);
}

# get data for revoking a certifikate
sub show_cert_revoke_dialog {
   my $self   = shift;
   my $passwd = shift;

   my ($dialog, $inputframe, $labelframe, $ent);

   $dialog = $self->{'mw'}->DialogBox( 
         -title          => 'Revoke Certifikate',
         -buttons        => ['Revoke', 'Cancel'],
         -default_button => 'Revoke'
         );

   $labelframe = $dialog->Frame( 
         -borderwidth => 5
         )->pack(-fill => 'x');
   $labelframe->Label(
         -text => 'Revoke Certifikate'
         )->pack(-side => 'top');

   $inputframe = $dialog->Frame( 
         -borderwidth => 10
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'CA Password:'
         )->pack(-side => 'left');
   $ent = $inputframe->Entry(
         -textvariable => $passwd,
         -show => '*'
         )->pack(-side => 'right');
   $ent->focus();
   $ent->icursor(0);

   $dialog->Show(1);
}

# get data for exporting a crl
sub show_crl_export_dialog {
   my $self    = shift;
   my $outfile = shift;
   my $format  = shift;
   my $passwd  = shift;
   my $days    = shift;
      $$format = 'PEM';

   my ($dialog, $inputframe, $labelframe, $radioframe, $passframe,
       $ent, $types);

   $types =
      [["PEM",     '.pem'],
       ["DER",     '.der'],
       ["TXT",     '.txt'],
       ["All files",  '*']
      ];

   $dialog = $self->{'mw'}->DialogBox( 
         -title          => 'Export CRL',
         -buttons        => ['Export', 'Cancel'],
         -default_button => 'Export');

   $labelframe = $dialog->Frame( 
         -borderwidth => 10
         )->pack(-fill => 'x');
   $labelframe->Label(
         -text => 'Export CRL'
         )->pack(-side => 'top');

   $passframe = $dialog->Frame( 
         -borderwidth => 3
         )->pack(-fill => 'x');
   $passframe->Label(
         -text => 'CA Password:'
         )->pack(-side => 'left');
   $ent = $passframe->Entry(
         -textvariable => $passwd,
         -show         => '*'
         )->pack(-side => 'left');
   $ent->focus();
   $ent->icursor(0);

   $inputframe = $dialog->Frame( 
         -borderwidth => 3
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'File:'
         )->pack(-side => 'left');
   $ent = $inputframe->Entry(
         -textvariable => $outfile
         );
   $inputframe->Button(
         -text => "Browse",
         -command => sub { 
            _save_file( $dialog, $ent, $types, '.pem') 
            }
         )->pack(-side => 'right');

   $ent->pack(-side => 'right');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'Valid for (Days):'
         )->pack(-side => 'left');
   $inputframe->Entry(
         -textvariable => $days
         )->pack(-side => 'right');

   $inputframe = $dialog->Frame(
         -borderwidth => 3
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'Export Format:'
         )->pack(
            -side   => 'left',
            -anchor => 'nw'
            );

   $radioframe = $inputframe->Frame(
         -borderwidth => 1
         )->pack(-side => 'right');
   $radioframe->Radiobutton(
         -text     => 'PEM',
         -value    => 'PEM',
         -command  => [ \&_changeformat, $outfile, $format ],
         -variable => $format
         )->pack(
            -side   => 'top',
            -anchor => 'nw'
            );
   $radioframe->Radiobutton(
         -text     => 'DER',
         -value    => 'DER',
         -command  => [  \&_changeformat, $outfile, $format],
         -variable => $format
         )->pack(
            -side   => 'top',
            -anchor => 'nw'
            );
   $radioframe->Radiobutton(
         -text     => 'TXT',
         -value    => 'TXT',
         -command  => [  \&_changeformat, $outfile, $format],
         -variable => $format
         )->pack(
            -side   => 'top',
            -anchor => 'nw'
            );

   $dialog->Show(1);
}

# get data for exporting a ca certificate
sub show_ca_export_dialog {
   my $self    = shift;
   my $outfile = shift;
   my $format  = shift;
      $$format = 'PEM';

   my ($dialog, $inputframe, $labelframe, $radioframe, $ent, $types);

   $types =
      [["PEM",     '.pem'],
       ["DER",     '.der'],
       ["Text",    '.txt'],
       ["All files",  '*']
      ];

   $dialog = $self->{'mw'}->DialogBox( 
         -title          => 'Export CA Certificate',
         -buttons        => ['Export', 'Cancel'],
         -default_button => 'Export');

   $labelframe = $dialog->Frame( 
         -borderwidth => 10
         )->pack(-fill => 'x');
   $labelframe->Label(
         -text => 'Export CA Certificate'
         )->pack(-side => 'top');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'File:'
         )->pack(-side => 'left');
   $ent = $inputframe->Entry(
         -textvariable => $outfile
         );
   $inputframe->Button(
         -text => "Browse",
         -command => sub { 
            _save_file( $dialog, $ent, $types, '.pem') 
            }
         )->pack(-side => 'right');
   $ent->insert('end', "/tmp/cacert.pem");
   $ent->pack(-side => 'right');

   $inputframe = $dialog->Frame(
         -borderwidth => 3
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'Export Format:'
         )->pack(
            -side   => 'left',
            -anchor => 'nw'
            );

   $radioframe = $inputframe->Frame(
         -borderwidth => 1
         )->pack(-side => 'right');
   $radioframe->Radiobutton(
         -text     => 'PEM',
         -value    => 'PEM',
         -variable => $format,
         -command  => [ \&_changeformat, $outfile, $format ],
         )->pack(
            -side   => 'top',
            -anchor => 'nw'
            );
   $radioframe->Radiobutton(
         -text     => 'DER',
         -value    => 'DER',
         -variable => $format,
         -command  => [ \&_changeformat, $outfile, $format ],
         )->pack(
            -side   => 'top',
            -anchor => 'nw'
            );
   $radioframe->Radiobutton(
         -text     => 'TXT',
         -value    => 'TXT',
         -variable => $format,
         -command  => [ \&_changeformat, $outfile, $format ],
         )->pack(
            -side   => 'top',
            -anchor => 'nw'
            );

   $dialog->Show(1);
}

# get data for exporting a key
sub show_key_export_dialog {
   my $self    = shift;
   my $outfile = shift;
   my $format  = shift;
   my $nopass  = shift;
      $$format = 'PEM';

   my ($dialog, $inputframe, $labelframe, $radioframe, $ent, 
       $types, $passbutton1, $passbutton2);

   $types =
      [["PEM",     '.pem'],
       ["DER",     '.der'],
       ["PKCS#12", '.p12'],
       ["All files",  '*']
      ];

   $dialog = $self->{'mw'}->DialogBox( 
         -title          => 'Export Key',
         -buttons        => ['Export', 'Cancel'],
         -default_button => 'Export');

   $labelframe = $dialog->Frame( 
         -borderwidth => 10
         )->pack(-fill => 'x');
   $labelframe->Label(
         -text => 'Export Key'
         )->pack(-side => 'top');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'File:'
         )->pack(-side => 'left');
   $ent = $inputframe->Entry(
         -textvariable => $outfile
         );
   $inputframe->Button(
         -text => "Browse",
         -command => sub { 
            _save_file( $dialog, $ent, $types, '.pem') 
            }
         )->pack(-side => 'right');
   $ent->insert('end', "/tmp/key.pem");
   $ent->pack(-side => 'right');

   $inputframe = $dialog->Frame(
         -borderwidth => 3
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'Export Format:'
         )->pack(
            -side   => 'left',
            -anchor => 'nw'
            );

   $radioframe = $inputframe->Frame(
         -borderwidth => 1
         )->pack(-side => 'right');

   # need to define earlier than the other buttons
   $inputframe = $dialog->Frame( 
         -borderwidth => 3
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'Without Passphrase (PEM)'
         )->pack(-side => 'left');

   $passbutton1 = $inputframe->Radiobutton(
         -variable => $nopass,
         -text     => 'Yes',
         -value    => 1
         );
   $passbutton2 = $inputframe->Radiobutton(
         -variable => $nopass,
         -text     => 'No',
         -value    => 0
         );

   $radioframe->Radiobutton(
         -text     => 'PEM (Key)',
         -value    => 'PEM',
         -variable => $format,
         -command  => [ \&_changeformat_key, $outfile, $format, 
                        $passbutton1, $passbutton2 ],
         )->pack(
            -side   => 'top',
            -anchor => 'nw'
            );
   $radioframe->Radiobutton(
         -text     => 'DER (Key without Passphrase)',
         -value    => 'DER',
         -variable => $format,
         -command  => [ \&_changeformat_key, $outfile, $format, 
                        $passbutton1, $passbutton2 ],
         )->pack(
            -side   => 'top',
            -anchor => 'nw'
            );
   $radioframe->Radiobutton(
         -text     => 'PKCS#12 (Certificate & Key)',
         -value    => 'P12',
         -variable => $format,
         -command  => [ \&_changeformat_key, $outfile, $format, 
                        $passbutton1, $passbutton2 ],
         )->pack(
            -side   => 'top',
            -anchor => 'nw'
            );

   $passbutton1->pack(-side => 'right');
   $passbutton2->pack(-side => 'right');


   $dialog->Show(1);
}

sub show_key_nopasswd_dialog {
   my $self    = shift;
   my $passwd  = shift;

   my ($dialog, $inputframe, $labelframe, $ent);

   $dialog = $self->{'mw'}->DialogBox( 
         -title          => 'Export Key without Passphrase',
         -buttons        => ['Export', 'Cancel'],
         -default_button => 'Export');

   $labelframe = $dialog->Frame( 
         -borderwidth => 10
         )->pack(-fill => 'x');
   $labelframe->Label(
         -text => 'Export Key without Passphrase'
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => 'I hope you know what you\'re doing'
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => ''
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => 'Key Passphrase (needed for decryption):'
         )->pack(-side => 'top');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'Passphrase:'
         )->pack(-side => 'left');
   $ent = $inputframe->Entry(
         -textvariable => $passwd,
         -show         => '*'
         )->pack(-side => 'right');;
   $ent->focus();
   $ent->icursor(0);

   $dialog->Show(1);
}

# get filename for importing a request
sub show_req_import_dialog {
   my $self   = shift;
   my $infile = shift;

   my ($dialog, $inputframe, $labelframe, $radioframe, $ent, $types);

   $types =
      [["PEM",     '.pem'],
       ["DER",     '.der'],
       ["Request", '.req'],
       ["All files",  '*']
      ];

   $dialog = $self->{'mw'}->DialogBox( 
         -title          => 'Import Request',
         -buttons        => ['Import', 'Cancel'],
         -default_button => 'Import');

   $labelframe = $dialog->Frame( 
         -borderwidth => 10
         )->pack(-fill => 'x');
   $labelframe->Label(
         -text => 'Import Request'
         )->pack(-side => 'top');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'File:'
         )->pack(-side => 'left');
   $ent = $inputframe->Entry(
         -textvariable => $infile
         );
   $inputframe->Button(
         -text => "Browse",
         -command => sub { 
            _open_file( $dialog, $ent, $types) 
            }
         )->pack(-side => 'right');
   $ent->pack(-side => 'right');

   $dialog->Show(1);
}

# get data for exporting a certificate
sub show_cert_export_dialog {
   my $self    = shift;
   my $outfile = shift;
   my $format  = shift;
      $$format = 'PEM';

   my ($dialog, $inputframe, $labelframe, $radioframe, $ent, $types);

   $types =
      [["PEM",     '.pem'],
       ["DER",     '.der'],
       ["PKCS#12", '.p12'],
       ["Text",    '.txt'],
       ["All files",  '*']
      ];

   $dialog = $self->{'mw'}->DialogBox( 
         -title          => 'Export Certificate',
         -buttons        => ['Export', 'Cancel'],
         -default_button => 'Export');

   $labelframe = $dialog->Frame( 
         -borderwidth => 10
         )->pack(-fill => 'x');
   $labelframe->Label(
         -text => 'Export Certificate'
         )->pack(-side => 'top');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'File:'
         )->pack(-side => 'left');
   $ent = $inputframe->Entry(
         -textvariable => $outfile
         );
   $inputframe->Button(
         -text => "Browse",
         -command => sub { 
            _save_file( $dialog, $ent, $types, '.pem') 
            }
         )->pack(-side => 'right');
   $ent->insert('end', "/tmp/cert.pem");
   $ent->pack(-side => 'right');

   $inputframe = $dialog->Frame(
         -borderwidth => 3
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'Export Format:'
         )->pack(
            -side   => 'left',
            -anchor => 'nw'
            );

   $radioframe = $inputframe->Frame(
         -borderwidth => 1
         )->pack(-side => 'right');
   $radioframe->Radiobutton(
         -text     => 'PEM (Certificate)',
         -value    => 'PEM',
         -variable => $format,
         -command  => [ \&_changeformat, $outfile, $format ],
         )->pack(
            -side   => 'top',
            -anchor => 'nw'
            );
   $radioframe->Radiobutton(
         -text     => 'DER (Certificate)',
         -value    => 'DER',
         -variable => $format,
         -command  => [ \&_changeformat, $outfile, $format ],
         )->pack(
            -side   => 'top',
            -anchor => 'nw'
            );
   $radioframe->Radiobutton(
         -text     => 'TXT (Certificate)',
         -value    => 'TXT',
         -variable => $format,
         -command  => [ \&_changeformat, $outfile, $format ],
         )->pack(
            -side   => 'top',
            -anchor => 'nw'
            );
   $radioframe->Radiobutton(
         -text     => 'PKCS#12 (Certificate & Key)',
         -value    => 'P12',
         -variable => $format,
         -command  => [ \&_changeformat, $outfile, $format ],
         )->pack(
            -side   => 'top',
            -anchor => 'nw'
            );

   $dialog->Show(1);
}

# get export passwd for pkcs#12
sub show_p12_export_dialog {
   my $self      = shift;
   my $expasswd  = shift;
   my $passwd    = shift;
   my $includeca = shift;

   my ($dialog, $inputframe, $labelframe, $ent);

   $dialog = $self->{'mw'}->DialogBox( 
         -title          => 'Export Password',
         -buttons        => ['OK', 'Cancel'],
         -default_button => 'OK'
         );

   $labelframe = $dialog->Frame( 
         -borderwidth => 5
         )->pack(-fill => 'x');
   $labelframe->Label(
         -text => 'Export to PKCS#12'
         )->pack(-side => 'top');

   $inputframe = $dialog->Frame( 
         -borderwidth => 8
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'Key Password:'
         )->pack(-side => 'left');
   $ent = $inputframe->Entry(
         -textvariable => $passwd,
         -show => '*'
         )->pack(-side => 'right');
   $ent->focus();
   $ent->icursor(0);

   $inputframe = $dialog->Frame( 
         -borderwidth => 8
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'Export Password:'
         )->pack(-side => 'left');
   $inputframe->Entry(
         -textvariable => $expasswd,
         -show => '*'
         )->pack(-side => 'right');

   $inputframe = $dialog->Frame( 
         -borderwidth => 8   
         )->pack(-fill => 'x');
   $inputframe->Label( 
         -text => 'Add CA Certificate'
         )->pack(-side => 'left');
   $inputframe->Radiobutton(
         -variable => $includeca,
         -text     => 'Yes',
         -value    => 1
         )->pack(-side => 'right');
   $inputframe->Radiobutton(
         -variable => $includeca,
         -text     => 'No',  
         -value    => 0
         )->pack(-side => 'right');

   $dialog->Show(1);
}

# get data for signing a request
sub show_req_sign_dialog {
   my $self      = shift;
   my $passwd    = shift;
   my $days      = shift;
   my $certtype  = shift;
      $$certtype = 'client';

   my($dialog, $inputframe, $labelframe, $ent);

   $dialog = $self->{'mw'}->DialogBox( 
         -title          => 'Sign Request',
         -buttons        => ['Sign', 'Cancel'],
         -default_button => 'Sign');

   $labelframe = $dialog->Frame( 
         -borderwidth => 10
         )->pack(-fill => 'x');
   $labelframe->Label(
         -text => 'Sign Request'
         )->pack(-side => 'top');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'CA Password:'
         )->pack(-side => 'left');
   $ent = $inputframe->Entry(
         -textvariable => $passwd,
         -show => '*'
         )->pack(-side => 'right');
   $ent->focus();
   $ent->icursor(0);

   $inputframe = $dialog->Frame( 
         -borderwidth => 3
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'Valid for (Days):'
         )->pack(-side => 'left');
   $inputframe->Entry(
         -textvariable => $days
         )->pack(-side => 'right');

   $dialog->Show(1);
}

# get data for creating a new CA
sub show_ca_dialog {
   my $self = shift;
   my $opt  = shift;

   my $dialog = $self->{'mw'}->DialogBox( 
         -title          => 'Create New CA',
         -buttons        => ['Create', 'Cancel'],
         -default_button => 'Create',
         -relief         => 'groove',
         -borderwidth    => 3
         );

   my $labelframe = $dialog->Frame( 
         -borderwidth => 10,
         )->pack(-fill => 'x');
   $labelframe->Label(
         -text       => 'Create a new CA',
         -font       => 'big',
         )->pack(-side => 'top');

   my $inputframe = $dialog->Frame( 
         -borderwidth => 3,
         )->pack(-fill => 'x');

   $inputframe->Label(
         -text       => 'Name (for local storage):',
         )->pack(-side => 'left');
   my $ent = $inputframe->Entry(
         -textvariable => \$opt->{'name'}
         )->pack(-side => 'right');
   $ent->focus();
   $ent->icursor(0);

   $inputframe = $dialog->Frame( 
         -borderwidth => 3,
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => '',
         )->pack(-side => 'left');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3,
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'Data for CA Certificate',
         )->pack(-fill => 'x');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3,
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'Common Name (for the CA):',
         )->pack(-side => 'left');
   $inputframe->Entry(
         -textvariable => \$opt->{'CN'}
         )->pack(-side => 'right');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3,
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text       => 'Country Name (2 letter code):',
         )->pack(-side => 'left');
   $inputframe->Entry(
         -textvariable => \$opt->{'C'}
         )->pack(-side => 'right');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3,
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text       => 'Password:',
         )->pack(-side => 'left');
   $inputframe->Entry(
         -textvariable => \$opt->{'passwd'},
         -show         => '*'
         )->pack(-side => 'right');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3,
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text       => 'Password (confirmation):',
         )->pack(-side => 'left');
   $inputframe->Entry(
         -textvariable => \$opt->{'passwd2'},
         -show         => '*'
         )->pack(-side => 'right');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3,
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'State or Province:',
         )->pack(-side => 'left');
   $inputframe->Entry(
         -textvariable => \$opt->{'ST'}
         )->pack(-side => 'right');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3,
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'Locality Name (eg, city):',
         )->pack(-side => 'left');
   $inputframe->Entry(
         -textvariable => \$opt->{'L'}
         )->pack(-side => 'right');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3,
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'Organization Name (eg, company):',
         )->pack(-side => 'left');
   $inputframe->Entry(
         -textvariable => \$opt->{'O'}
         )->pack(-side => 'right');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3,
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'Organizational Unit Name (eg, section):',
         )->pack(-side => 'left');
   $inputframe->Entry(
         -textvariable => \$opt->{'OU'}
         )->pack(-side => 'right');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3,
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'Email Address:',
         )->pack(-side => 'left');
   $inputframe->Entry(
         -textvariable => \$opt->{'EMAIL'}
         )->pack(-side => 'right');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3,
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'Valid for (days):',
         )->pack(-side => 'left');
   $inputframe->Entry(
         -textvariable => \$opt->{'days'}
         )->pack(-side => 'right');

   $inputframe = $dialog->Frame( 
         -borderwidth => 3
         )->pack(-fill => 'x');
   $inputframe->Label(
         -text => 'Keylength'
         )->pack(-side => 'left');
   $inputframe->Radiobutton( 
         -text  => '1024',
         -value => 1024,
         -variable => \$opt->{'bits'}
         )->pack(-side => 'right');
   $inputframe->Radiobutton(
         -text  => '2048',
         -value => 2048,
         -variable => \$opt->{'bits'}
         )->pack(-side => 'right');
   $inputframe->Radiobutton(
         -text  => '4096',
         -value => 4096,
         -variable => \$opt->{'bits'}
         )->pack(-side => 'right');

   $dialog->Show(1);
}

### subroutines for pop-up boxes
sub show_help {
   my $self = shift;

   $self->print_warning("You are kidding, are you??");

   return;
}

sub about {
   my $self = shift;

   my $d = $self->{'mw'}->DialogBox(
         -title => 'About',
         -buttons => ['OK']
         );
   
   my $f = $d->Frame(
         -borderwidth => 3,
         -relief      => 'groove'
         )->pack(
            -fill   => 'both',
            -side   => 'top',
            -expand => 1
            );
   
   $f = $f->Frame(
         -borderwidth => 10,
         -background  => 'white'
         )->pack(
            -fill   => 'both',
            -side   => 'top',
            -expand => 1
            );

   $f->Label(
         -text        => "",
         -font        => 'small',
         -background  => 'white'
         )->pack();
   $f->Label(
         -text        => "TinyCA Version $self->{'version'}",
         -background  => 'white'
         )->pack();
   $f->Label(
         -text        => "",
         -font        => 'small',
         -background  => 'white'
         )->pack();
   $f->Label(
         -text        => "Stephan Martin <sm\@sm-zone.net>",
         -font        => 'small',
         -background  => 'white'
         )->pack();
   $f->Label(
         -text        => "",
         -font        => 'small',
         -background  => 'white'
         )->pack();
   $f->Label(
         -text        => "http://tinyca.sm-zone.net/",
         -font        => 'small',
         -background  => 'white'
         )->pack();
   $f->Label(
         -text        => "",
         -font        => 'small',
         -background  => 'white'
         )->pack();

   $d->Show(1);
   
   return;
}

sub print_error {
   my $self = shift;
   my $t    = shift;
   
   my $d = $self->{'mw'}->messageBox(
	 -title => 'Error',
	 -type => 'OK',
	 -message => $t,
	 -icon => 'error',
	 -font => 'medium',
	 );

   $self->exit_clean(1);
}

sub print_warning {
   my $self = shift;
   my $t    = shift;
   
   my $d = $self->{'mw'}->messageBox(
	 -title => 'Warning',
	 -type => 'OK',
	 -message => $t,
	 -icon => 'warning',
	 -font => 'medium',
	 );
}

sub print_info {
   my $self = shift;
   my $t    = shift;
   
   my $d = $self->{'mw'}->messageBox(
	 -title => 'Info',
	 -type => 'OK',
	 -message => $t,
	 -icon => 'info',
	 -font => 'medium',
	 );
}

# get confirmation for deleting a request
sub show_req_del_confirm {
   my $self = shift;

   my $dialog = $self->{'mw'}->DialogBox( 
         -title          => 'Delete Request',
         -buttons        => ['Delete', 'Cancel'],
         -default_button => 'Cancel');

   my $labelframe = $dialog->Frame( 
         -borderwidth => 10
         )->pack(-fill => 'x');
   $labelframe->Label(
         -text => 'Delete Request'
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => 'Do you really want to delete the selected request?'
         )->pack(-side => 'top');

   $dialog->Show(1);
}

# ask if request shall be deleted after signing
sub show_req_del_question {
   my $self = shift;

   my $dialog = $self->{'mw'}->DialogBox( 
         -title          => 'Delete Request',
         -buttons        => ['Yes', 'No'],
         -default_button => 'No'
         );

   my $labelframe = $dialog->Frame( 
         -borderwidth => 10
         )->pack(-fill => 'x');
   $labelframe->Label(
         -text => 'The Request is not needed anymore.'
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => 'Do you want to delete it?'
         )->pack(-side => 'top');

   $dialog->Show(1);
}

# ask if key shall be deleted after revoking
sub show_key_del_question {
   my $self = shift;

   my $dialog = $self->{'mw'}->DialogBox( 
         -title          => 'Delete Key and Request',
         -buttons        => ['Yes', 'No'],
         -default_button => 'No'
         );

   my $labelframe = $dialog->Frame( 
         -borderwidth => 10
         )->pack(-fill => 'x');
   $labelframe->Label(
         -text => 'The Request and the Key are not needed anymore.'
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => 'Do you want to delete them?'
         )->pack(-side => 'top');

   $dialog->Show(1);
}

# show warning - overwrite key
sub show_req_overwrite_warning {
   my $self = shift;

   my $dialog = $self->{'mw'}->DialogBox( 
         -title          => 'Warning',
         -buttons        => ['Overwrite', 'Cancel'],
         -default_button => 'Overwrite'
         );

   my $labelframe = $dialog->Frame( 
         -borderwidth => 10
         )->pack(-fill => 'x');
   $labelframe->Label(
         -text => 'Warning',
         -font => 'big'
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => 'The Key or the Request is already existing!',
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => 'If it\'s not expired or revoked'
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => 'you won\'t be able'
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => 'to sign this request!!'
         )->pack(-side => 'top');

   $dialog->Show(1);
}

# show warning - certificate expiration date
sub show_req_date_warning {
   my $self = shift;

   my $dialog = $self->{'mw'}->DialogBox( 
         -title          => 'Warning',
         -buttons        => ['Continue', 'Cancel'],
         -default_button => 'Cancel'
         );

   my $labelframe = $dialog->Frame( 
         -borderwidth => 10
         )->pack(-fill => 'x');
   $labelframe->Label(
         -text => 'Warning',
         -font => 'big'
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => 'The Certificate will be longer valid than your CA!'
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => 'This may cause problems with some software!!'
         )->pack(-side => 'top');

   $dialog->Show(1);
}

# get confirmation for deleting a key
sub show_key_del_confirm {
   my $self = shift;

   my $dialog = $self->{'mw'}->DialogBox( 
         -title          => 'Delete Key',
         -buttons        => ['Delete', 'Cancel'],
         -default_button => 'Cancel'
         );

   my $labelframe = $dialog->Frame( 
         -borderwidth => 10
         )->pack(-fill => 'x');
   $labelframe->Label(
         -text => 'Delete Key'
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => 'Do you really want to delete the selected key?'
         )->pack(-side => 'top');

   $dialog->Show(1);
}

# get confirmation for overwriting certificate
sub show_cert_overwrite_confirm {
   my $self = shift;

   my $dialog = $self->{'mw'}->DialogBox( 
         -title          => 'Overwrite Certificate',
         -buttons        => ['Overwrite', 'Cancel'],
         -default_button => 'Cancel'
         );

   my $labelframe = $dialog->Frame( 
         -borderwidth => 10
         )->pack(-fill => 'x');
   $labelframe->Label(
         -text => 'There seems to be an certificate already.'
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => 'Signing will fail if it\'s not revoked or expired!'
         )->pack(-side => 'top');

   $dialog->Show(1);
}

# main screen for configuration
sub show_config_dialog {
   my $self = shift;
   my $name = shift;

   if(not defined($name)) {
      $name = $self->{'CA'}->{'actca'};
   }
   if(not defined($name)) {
      $self->print_warning("Can't get CA name");
      return;
   }

   $self->{'CONFIG'}->init_config($self, $name);

   my $dialog = $self->{'mw'}->DialogBox( 
         -title          => 'Initial Configuration',
         -buttons        => ['Create', 'Cancel'],
         -default_button => 'Create'
         );

   my $labelframe = $dialog->Frame( 
         -borderwidth => 10
         )->pack(-fill => 'x');
   $labelframe->Label(
         -text => 'Initial Configuration',
         -font => 'big'
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => 'Only change these options, if you '
                 .'really know, what you are doing!!'
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => 'You should be aware, that some options'
                 .' may brake crappy software!!'
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => 'If you are unsure: leave the defaults untouched'
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => ''
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => 'Options used for creating the CA Certificate:'
         )->pack(-side => 'top');
   $labelframe->Button(
         -text    => 'CA Options',
         -command => sub { $self->{'CONFIG'}->config_ca($self, $name) },
         )->pack(
            -side   => 'top',
            -fill   => 'x',
            -expand => 1
            );
   $labelframe->Label(
         -text => ''
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => 'Options used for creating Server Certificates:'
         )->pack(-side => 'top');
   $labelframe->Button(
         -text    => 'Server Certificate Options',
         -command => sub { $self->{'CONFIG'}->config_server($self, $name) },
         )->pack(
            -side   => 'top',
            -fill   => 'x',
            -expand => 1
            );
   $labelframe->Label(
         -text => ''
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => 'Options used for creating Client Certificates:'
         )->pack(-side => 'top');
   $labelframe->Button(
         -text    => 'Client Certificate Options',
         -command => sub { $self->{'CONFIG'}->config_client($self, $name) },
         )->pack(
            -side   => 'top',
            -fill   => 'x',
            -expand => 1
            );
   $labelframe->Label(
         -text => ''
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => 'Options used for creating Revocation Lists:'
         )->pack(-side => 'top');
   $labelframe->Button(
         -text    => 'Revocation List Options',
         -command => sub { $self->{'CONFIG'}->config_crl($self, $name) },
         )->pack(
            -side   => 'top',
            -fill   => 'x',
            -expand => 1
            );
         

   $dialog->Show(1);
}

# configuration for CA
sub show_config_ca {
   my $self = shift;
   my $name = shift;

   my(@options, $key);

   @options = qw(
         authorityKeyIdentifier
         basicConstraints
         keyUsage
         nsCertType
         issuerAltName
         nsComment
         crlDistributionPoints
         nsCaRevocationUrl
         nsCaPolicyUrl
         nsRevocationUrl
         nsPolicyUrl
         );
   
   if(not defined($name)) {
      $name = $self->{'CA'}->{'actca'};
   }
   if(not defined($name)) {
      $self->print_warning("Can't get CA name");
      return;
   }

   $self->{'CONFIG'}->init_config($self, $name);

   my $dialog = $self->{'mw'}->DialogBox( 
         -title          => 'CA Configuration',
         -buttons        => ['OK', 'Cancel'],
         -default_button => 'OK'
         );

   $dialog->focus();

   my $labelframe = $dialog->Frame( 
         -borderwidth => 10
         )->pack(-fill => 'x');
   $labelframe->Label(
         -text => 'CA Configuration',
         -font => 'big'
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => 'These Options are used for creating'
                 .' the CA certificate'
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => 'Multiple Values can be separated by ","'
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => ''
         )->pack(-side => 'top');

   foreach $key (@options) {
      $labelframe->Label(
            -text => "$key"
            )->pack(-side => 'top');
      $labelframe->Entry(
            -textvariable => \$self->{'CONFIG'}->{'v3_ca'}->{$key}
            )->pack(
               -side   => 'top',
               -fill   => 'x',
               -expand => 1
               );
   }

   $dialog->Show(1);
}

# configuration for server
sub show_config_server {
   my $self = shift;
   my $name = shift;

   my(@options, @options_ca, $key);

   @options = qw(
         nsCertType
         nsComment
         authorityKeyIdentifier
         subjectAltName
         issuerAltName
         crlDistributionPoints
         nsCaRevocationUrl
         nsBaseUrl
         nsRevocationUrl
         nsRenewalUrl
         nsCaPolicyUrl
         );


   @options_ca = qw(
         default_days
         );
   
   if(not defined($name)) {
      $name = $self->{'CA'}->{'actca'};
   }
   if(not defined($name)) {
      $self->print_warning("Can't get CA name");
      return;
   }

   $self->{'CONFIG'}->init_config($self, $name);

   my $dialog = $self->{'mw'}->DialogBox( 
         -title          => 'Server Certificate Configuration',
         -buttons        => ['OK', 'Cancel'],
         -default_button => 'OK'
         );

   $dialog->focus();

   my $labelframe = $dialog->Frame( 
         -borderwidth => 10
         )->pack(-fill => 'x');
   $labelframe->Label(
         -text => 'Server Certificate Configuration',
         -font => 'big'
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => 'These Options are used for creating'
                 .' Server Certificates'
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => 'Multiple Values can be separated by ","'
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => ''
         )->pack(-side => 'top');

   foreach $key (@options) {
      $labelframe->Label(
            -text => "$key"
            )->pack(-side => 'top');
      $labelframe->Entry(
            -textvariable => \$self->{'CONFIG'}->{'server_cert'}->{$key}
            )->pack(
               -side   => 'top',
               -fill   => 'x',
               -expand => 1
               );
   }

   foreach $key (@options_ca) {
      $labelframe->Label(
            -text => "$key"
            )->pack(-side => 'top');
      $labelframe->Entry(
            -textvariable => \$self->{'CONFIG'}->{'server_ca'}->{$key}
            )->pack(
               -side   => 'top',
               -fill   => 'x',
               -expand => 1
               );
   }

   $dialog->Show(1);
}

# configuration for crls
sub show_config_crl {
   my $self = shift;
   my $name = shift;

   my(@options, $key);

   @options = qw(
         default_crl_days
         );
   
   if(not defined($name)) {
      $name = $self->{'CA'}->{'actca'};
   }
   if(not defined($name)) {
      $self->print_warning("Can't get CA name");
      return;
   }

   $self->{'CONFIG'}->init_config($self, $name);

   my $dialog = $self->{'mw'}->DialogBox( 
         -title          => 'Revocation List Configuration',
         -buttons        => ['OK', 'Cancel'],
         -default_button => 'OK'
         );

   $dialog->focus();

   my $labelframe = $dialog->Frame( 
         -borderwidth => 10
         )->pack(-fill => 'x');
   $labelframe->Label(
         -text => 'Revocation List Configuration',
         -font => 'big'
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => 'These Options are used for creating'
                 .' Certificate Revocation Lists'
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => 'Multiple Values can be separated by ","'
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => ''
         )->pack(-side => 'top');

   foreach $key (@options) {
      $labelframe->Label(
            -text => "$key"
            )->pack(-side => 'top');
      $labelframe->Entry(
            -textvariable => \$self->{'CONFIG'}->{'server_ca'}->{$key}
            )->pack(
               -side   => 'top',
               -fill   => 'x',
               -expand => 1
               );
   }

   $dialog->Show(1);
}

# configuration for clients
sub show_config_client {
   my $self = shift;
   my $name = shift;

   my(@options, @options_ca, $key);

   @options = qw(
         nsCertType
         nsComment
         authorityKeyIdentifier
         subjectAltName
         issuerAltName
         crlDistributionPoints
         nsCaRevocationUrl
         nsBaseUrl
         nsRevocationUrl
         nsRenewalUrl
         nsCaPolicyUrl
         );

   @options_ca = qw(
         default_days
         );
   
   if(not defined($name)) {
      $name = $self->{'CA'}->{'actca'};
   }
   if(not defined($name)) {
      $self->print_warning("Can't get CA name");
      return;
   }

   $self->{'CONFIG'}->init_config($self, $name);

   my $dialog = $self->{'mw'}->DialogBox( 
         -title          => 'Client Certificate Configuration',
         -buttons        => ['OK', 'Cancel'],
         -default_button => 'OK'
         );

   $dialog->focus();

   my $labelframe = $dialog->Frame( 
         -borderwidth => 10
         )->pack(-fill => 'x');
   $labelframe->Label(
         -text => 'Client Certificate Configuration',
         -font => 'big'
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => 'These Options are used for creating'
                 .' Client Certificates'
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => 'Multiple Values can be separated by ","'
         )->pack(-side => 'top');
   $labelframe->Label(
         -text => ''
         )->pack(-side => 'top');

   foreach $key (@options) {
      $labelframe->Label(
            -text => "$key"
            )->pack(-side => 'top');
      $labelframe->Entry(
            -textvariable => \$self->{'CONFIG'}->{'client_cert'}->{$key}
            )->pack(
               -side   => 'top',
               -fill   => 'x',
               -expand => 1
               );
   }

   foreach $key (@options_ca) {
      $labelframe->Label(
            -text => "$key"
            )->pack(-side => 'top');
      $labelframe->Entry(
            -textvariable => \$self->{'CONFIG'}->{'client_ca'}->{$key}
            )->pack(
               -side   => 'top',
               -fill   => 'x',
               -expand => 1
               );
   }

   $dialog->Show(1);
}

# replace pem|der|txt suffix of filename
# parameters:
#	- reference to the variable that holds the filename
#	- reference to variable with new suffix
sub _changeformat($$) {
   my $outfileref = shift;
   my $formatref  = shift;

   if($$outfileref =~ s/\.(pem|der|txt|p12)$//i) {
      $$outfileref .= "." . lc $$formatref;
   }
}

# replace pem|der|txt suffix of filename and change status
# parameters:
#	- reference to the variable that holds the filename
#	- reference to variable with new suffix
#	- references to the buttons
sub _changeformat_key($$$$$) {
   my $outfileref  = shift;
   my $formatref   = shift;
   my $passbutton1 = shift;
   my $passbutton2 = shift;

   if($$outfileref =~ s/\.(pem|der|txt|p12)$//i) {
      $$outfileref .= "." . lc $$formatref;
   }
   
   if($$formatref eq "PEM") {
      $passbutton1->configure( -state => "normal");
      $passbutton2->configure( -state => "normal");
   } elsif ($$formatref eq "DER") {
      $passbutton1->configure( -state => "disabled");
      $passbutton2->configure( -state => "disabled");
   } elsif ($$formatref eq "P12") {
      $passbutton1->configure( -state => "disabled");
      $passbutton2->configure( -state => "disabled");
   }

      
}

# file dialog for opening files
sub _open_file {
   my $top   = shift;
   my $ent   = shift;
   my $types = shift;

   my $file = $top->getOpenFile(
         -title      => 'Import File',
         -filetypes  => $types,
         -initialdir => '/'
         );

   $ent->delete(0, 'end');
   $ent->insert(0, $file);
   $ent->xview('end');
}

# file dialog for saving files
sub _save_file {
   my $top     = shift;
   my $ent     = shift;
   my $types   = shift;
   my $default = shift;

   my $file = $top->getSaveFile(
         -title      => 'Export File',
         -filetypes  => $types
         );

   $ent->delete(0, 'end');
   $ent->insert(0, $file);
   $ent->xview('end');
}

# generate line to display requests
sub _gen_reqline {
   my $in = shift;

   my @line = split(/:/, $in);

   my $ret = $line[0];

   $ret .= " "x(25 - length($ret));

   for(my $i = 1; $line[$i]; $i++) {
      $ret .= " $line[$i]";
   }

   return($ret);
}

sub _cert_line {
   my $l = shift;
   my $r = shift;
   my $k = shift;
   my $v = shift;
   
   $l->Label( 
         -text => $k,
         -background  => 'white',
         -font        => 'small'
         )->pack(
            -fill   => 'y',
            -side   => 'top',
            -anchor => 'nw'
            );
   $r->Label(
         -text => $v,
         -background  => 'white',
         -font        => 'small'
         )->pack(
            -fill   => 'y',
            -side   => 'top',
            -anchor => 'ne'
            );

   return;
}

sub _list_header {
   my $w = shift;
   my $i = shift;
   my $t = shift;

   $w->header( 
         'create',
         $i,
         -text => $t
         );

   return;
}

1

# 
# $Log: GUI.pm,v $
# Revision 1.12  2003/06/23 21:31:17  sm
# changed version
#
# Revision 1.11  2003/06/23 21:16:29  sm
# automatically change filename and buttonstatus during export
#
# Revision 1.10  2003/06/23 20:11:29  sm
# some new texts from ludwig.nussel@suse.de
#
# Revision 1.9  2003/06/19 21:46:43  sm
# change button status dynamically
#
# Revision 1.7  2003/06/19 13:52:03  sm
# made default_crl_days configurable and some more usability stuff
#
# Revision 1.4  2002/10/07 17:33:58  sm
# added horizontal scrollbar to listings
# modified some dialogs (open, delete,...)
#
# Revision 1.3  2002/10/04 15:15:46  sm
# increased version
#
# Revision 1.2  2002/09/27 19:54:50  sm
# Increased version
#
# 
